@@ -15,9 +15,13 @@ BEGIN {
   *_HAVE_IS_UTF8 = defined &utf8::is_utf8 ? sub(){1} : sub(){0};
   *_HAVE_PERLSTRING = defined &B::perlstring ? sub(){1} : sub(){0};
   *_BAD_BACKSLASH_ESCAPE = _HAVE_PERLSTRING() && "$]" == 5.010_000 ? sub(){1} : sub(){0};
+  *_HAVE_HEX_FLOAT = "$]" >= 5.022 ? sub(){1} : sub(){0};
+
+  my $precision = length 1/9;
+  *_MAX_FLOAT_PRECISION = sub(){$precision};
 }
 
-our $VERSION = '2.006_002';
+our $VERSION = '2.006_004';
 $VERSION = eval $VERSION;
 
 our @EXPORT = qw(quote_sub unquote_sub quoted_from_sub qsub);
@@ -50,20 +54,27 @@ sub quotify {
     && 0 + $value eq $value
   ) ? (
     $value != $value ? (
-      $value eq -CORE::sin(9**9**9)
-        ? '(-CORE::sin(9**9**9))' # -nan
-        : 'CORE::sin(9**9**9)'    # nan
+      $value eq -(9**9**9*0)
+        ? '(-(9**9**9*0))' # -nan
+        : '(9**9**9*0)'    # nan
     )
     : $value == 9**9**9 ? '(9**9**9)'      # inf
     : $value == -9**9**9 ? '(-9**9**9)'    # -inf
-    : int($value) == $value ? $value       # integer
     : do {
-      my $float = sprintf('%.20f', $value);
-      $float =~ s/(\.[0-9]+?)0+\z/$1/;
-      $float;
+      my $float = $value;
+      for my $precision (0 .. _MAX_FLOAT_PRECISION) {
+        $float = sprintf '%.'.$precision.'g', $value;
+        last
+          if $float == $value;
+
+        if (_HAVE_HEX_FLOAT and $precision == _MAX_FLOAT_PRECISION) {
+          $float = sprintf "%a", $value;
+        }
+      }
+      "$float";
     }
   )
-  : !length($value) && eval { use warnings 'FATAL' => 'numeric'; $value == 0 } ? '(!1)' # false
+  : !length($value) && length( (my $dummy2 = '') & $value ) ? '(!1)' # false
   : _BAD_BACKSLASH_ESCAPE && _HAVE_IS_UTF8 && utf8::is_utf8($value) ? do {
     $value =~ s/(["\$\@\\[:cntrl:]]|[^\x00-\x7f])/
       $escape{$1} || sprintf('\x{%x}', ord($1))
